@@ -1,5 +1,5 @@
 // ***************************************************************** -*- C++ -*-
-// exifprint.cpp, $Rev: 1686 $
+// exifprint.cpp, $Rev: 2014 $
 // Sample program to print the Exif metadata of an image
 
 #include <exiv2/image.hpp>
@@ -28,12 +28,13 @@ try {
     }
     Exiv2::ExifData::const_iterator end = exifData.end();
     for (Exiv2::ExifData::const_iterator i = exifData.begin(); i != end; ++i) {
+        const char* tn = i->typeName();
         std::cout << std::setw(44) << std::setfill(' ') << std::left
                   << i->key() << " "
                   << "0x" << std::setw(4) << std::setfill('0') << std::right
                   << std::hex << i->tag() << " "
                   << std::setw(9) << std::setfill(' ') << std::left
-                  << i->typeName() << " "
+                  << (tn ? tn : "Unknown") << " "
                   << std::dec << std::setw(3)
                   << std::setfill(' ') << std::right
                   << i->count() << "  "
@@ -43,7 +44,9 @@ try {
 
     return 0;
 }
-catch (Exiv2::AnyError& e) {
-    std::cout << "Caught Exiv2 exception '" << e << "'\n";
+//catch (std::exception& e) {
+//catch (Exiv2::AnyError& e) {
+catch (Exiv2::Error& e) {
+    std::cout << "Caught Exiv2 exception '" << e.what() << "'\n";
     return -1;
 }
