@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      basicio.cpp
-  Version:   $Rev: 1983 $
+  Version:   $Rev: 2045 $
   Author(s): Brad Schick (brad) <brad@robotbattle.com>
   History:   04-Dec-04, brad: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: basicio.cpp 1983 2009-12-29 15:24:40Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: basicio.cpp 2045 2010-04-03 07:53:30Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -98,7 +98,7 @@ namespace Exiv2 {
         // DATA
         std::string path_;              //!< (Standard) path
 #ifdef EXV_UNICODE_PATH
-        std::wstring wpath_;            //!< Unicode path 
+        std::wstring wpath_;            //!< Unicode path
         WpMode wpMode_;                 //!< Indicates which path is in use
 #endif
         std::string openMode_;          //!< File open mode
@@ -109,7 +109,7 @@ namespace Exiv2 {
         HANDLE hFile_;                  //!< Duplicated fd
         HANDLE hMap_;                   //!< Handle from CreateFileMapping
 #endif
-        byte* pMappedArea_;             //!< Pointer to the memory-mapped area 
+        byte* pMappedArea_;             //!< Pointer to the memory-mapped area
         size_t mappedLength_;           //!< Size of the memory-mapped area
         bool isMalloced_;               //!< Is the mapped area allocated?
         bool isWriteable_;              //!< Can the mapped area be written to?
@@ -208,7 +208,15 @@ namespace Exiv2 {
         }
         openMode_ = "r+b";
         opMode_ = opSeek;
-        fp_ = std::fopen(path_.c_str(), openMode_.c_str());
+#ifdef EXV_UNICODE_PATH
+        if (wpMode_ == wpUnicode) {
+            fp_ = ::_wfopen(wpath_.c_str(), s2ws(openMode_).c_str());
+        }
+        else
+#endif
+        {
+            fp_ = std::fopen(path_.c_str(), openMode_.c_str());
+        }
         if (!fp_) return 1;
         return std::fseek(fp_, offset, SEEK_SET);
     } // FileIo::Impl::switchMode
@@ -293,11 +301,29 @@ namespace Exiv2 {
     {
         assert(p_->fp_ != 0);
         if (munmap() != 0) {
-            throw Error(2, path(), strError(), "munmap");
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(2, wpath(), strError().c_str(), "munmap");
+            }
+            else
+#endif
+            {
+                throw Error(2, path(), strError(), "munmap");
+            }
         }
         p_->mappedLength_ = size();
         p_->isWriteable_ = isWriteable;
-        if (p_->isWriteable_ && p_->switchMode(Impl::opWrite) != 0) return 0;
+        if (p_->isWriteable_ && p_->switchMode(Impl::opWrite) != 0) {
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(16, wpath(), strError().c_str());
+            }
+            else
+#endif
+            {
+                throw Error(16, path(), strError());
+            }
+        }
 #if defined EXV_HAVE_MMAP && defined EXV_HAVE_MUNMAP
         int prot = PROT_READ;
         if (p_->isWriteable_) {
@@ -305,7 +331,15 @@ namespace Exiv2 {
         }
         void* rc = ::mmap(0, p_->mappedLength_, prot, MAP_SHARED, fileno(p_->fp_), 0);
         if (MAP_FAILED == rc) {
-            throw Error(2, path(), strError(), "mmap");
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(2, wpath(), strError().c_str(), "mmap");
+            }
+            else
+#endif
+            {
+                throw Error(2, path(), strError(), "mmap");
+            }
         }
         p_->pMappedArea_ = static_cast<byte*>(rc);
 
@@ -326,25 +360,77 @@ namespace Exiv2 {
         HANDLE hPh = GetCurrentProcess();
         HANDLE hFd = (HANDLE)_get_osfhandle(fileno(p_->fp_));
         if (hFd == INVALID_HANDLE_VALUE) {
-            throw Error(2, path(), "MSG1", "_get_osfhandle");
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(2, wpath(), "MSG1", "_get_osfhandle");
+            }
+            else
+#endif
+            {
+                throw Error(2, path(), "MSG1", "_get_osfhandle");
+            }
         }
         if (!DuplicateHandle(hPh, hFd, hPh, &p_->hFile_, 0, false, DUPLICATE_SAME_ACCESS)) {
-            throw Error(2, path(), "MSG2", "DuplicateHandle");
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(2, wpath(), "MSG2", "DuplicateHandle");
+            }
+            else
+#endif
+            {
+                throw Error(2, path(), "MSG2", "DuplicateHandle");
+            }
         }
         p_->hMap_ = CreateFileMapping(p_->hFile_, 0, flProtect, 0, p_->mappedLength_, 0);
         if (p_->hMap_ == 0 ) {
-            throw Error(2, path(), "MSG3", "CreateFileMapping");
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(2, wpath(), "MSG3", "CreateFileMapping");
+            }
+            else
+#endif
+            {
+                throw Error(2, path(), "MSG3", "CreateFileMapping");
+            }
         }
         void* rc = MapViewOfFile(p_->hMap_, dwAccess, 0, 0, 0);
         if (rc == 0) {
-            throw Error(2, path(), "MSG4", "CreateFileMapping");
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(2, wpath(), "MSG4", "CreateFileMapping");
+            }
+            else
+#endif
+            {
+                throw Error(2, path(), "MSG4", "CreateFileMapping");
+            }
         }
         p_->pMappedArea_ = static_cast<byte*>(rc);
 #else
         // Workaround for platforms without mmap: Read the file into memory
         DataBuf buf(static_cast<long>(p_->mappedLength_));
-        read(buf.pData_, buf.size_);
-        if (error() || eof()) throw Error(2, path(), strError(), "FileIo::mmap");
+        if (read(buf.pData_, buf.size_) != buf.size_) {
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(2, wpath(), strError().c_str(), "FileIo::read");
+            }
+            else
+#endif
+            {
+                throw Error(2, path(), strError(), "FileIo::read");
+            }
+        }
+        if (error() || eof()) {
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(2, wpath(), strError().c_str(), "FileIo::mmap");
+            }
+            else
+#endif
+            {
+                throw Error(2, path(), strError(), "FileIo::mmap");
+            }
+        }
         p_->pMappedArea_ = buf.release().first;
         p_->isMalloced_ = true;
 #endif
@@ -374,7 +460,15 @@ namespace Exiv2 {
                 fileIo = std::auto_ptr<FileIo>(new FileIo(tmpname));
             }
             if (fileIo->open("w+b") != 0) {
-                throw Error(10, path(), "w+b", strError());
+#ifdef EXV_UNICODE_PATH
+                if (p_->wpMode_ == Impl::wpUnicode) {
+                    throw WError(10, wpath(), "w+b", strError().c_str());
+                }
+                else
+#endif
+                {
+                    throw Error(10, path(), "w+b", strError());
+                }
             }
             basicIo = fileIo;
         }
@@ -436,22 +530,34 @@ namespace Exiv2 {
                 {
                     ::remove(fileIo->path().c_str());
                 }
-                throw Error(10, path(), "w+b", strError());
+#ifdef EXV_UNICODE_PATH
+                if (p_->wpMode_ == Impl::wpUnicode) {
+                    throw WError(10, wpath(), "w+b", strError().c_str());
+                }
+                else
+#endif
+                {
+                    throw Error(10, path(), "w+b", strError());
+                }
             }
             close();
 
             bool statOk = true;
             mode_t origStMode = 0;
+            std::string spf;
             char* pf = 0;
 #ifdef EXV_UNICODE_PATH
+            std::wstring wspf;
             wchar_t* wpf = 0;
             if (p_->wpMode_ == Impl::wpUnicode) {
-                wpf = const_cast<wchar_t*>(wpath().c_str());
+                wspf = wpath();
+                wpf = const_cast<wchar_t*>(wspf.c_str());
             }
             else
 #endif
             {
-                pf = const_cast<char*>(path().c_str());
+                spf = path();
+                pf = const_cast<char*>(spf.c_str());
             }
 
             // Get the permissions of the file, or linked-to file, on platforms which have lstat
@@ -498,10 +604,10 @@ namespace Exiv2 {
 #ifdef EXV_UNICODE_PATH
             if (p_->wpMode_ == Impl::wpUnicode) {
                 if (fileExists(wpf) && ::_wremove(wpf) != 0) {
-                    throw Error(2, wpf, strError(), "::_wremove");
+                    throw WError(2, wpf, strError().c_str(), "::_wremove");
                 }
                 if (::_wrename(fileIo->wpath().c_str(), wpf) == -1) {
-                    throw Error(17, ws2s(fileIo->wpath()), wpf, strError());
+                    throw WError(17, fileIo->wpath(), wpf, strError().c_str());
                 }
                 ::_wremove(fileIo->wpath().c_str());
                 // Check permissions of new file
@@ -552,10 +658,26 @@ namespace Exiv2 {
         else {
             // Generic handling, reopen both to reset to start
             if (open("w+b") != 0) {
-                throw Error(10, path(), "w+b", strError());
+#ifdef EXV_UNICODE_PATH
+                if (p_->wpMode_ == Impl::wpUnicode) {
+                    throw WError(10, wpath(), "w+b", strError().c_str());
+                }
+                else
+#endif
+                {
+                    throw Error(10, path(), "w+b", strError());
+                }
             }
             if (src.open() != 0) {
-                throw Error(9, src.path(), strError());
+#ifdef EXV_UNICODE_PATH
+                if (p_->wpMode_ == Impl::wpUnicode) {
+                    throw WError(9, src.wpath(), strError().c_str());
+                }
+                else
+#endif
+                {
+                    throw Error(9, src.path(), strError());
+                }
             }
             write(src);
             src.close();
@@ -563,13 +685,31 @@ namespace Exiv2 {
 
         if (wasOpen) {
             if (open(lastMode) != 0) {
-                throw Error(10, path(), lastMode, strError());
+#ifdef EXV_UNICODE_PATH
+                if (p_->wpMode_ == Impl::wpUnicode) {
+                    throw WError(10, wpath(), lastMode.c_str(), strError().c_str());
+                }
+                else
+#endif
+                {
+                    throw Error(10, path(), lastMode, strError());
+                }
             }
         }
         else close();
 
-        if (error() || src.error()) throw Error(18, path(), strError());
-    }
+        if (error() || src.error()) {
+#ifdef EXV_UNICODE_PATH
+            if (p_->wpMode_ == Impl::wpUnicode) {
+                throw WError(18, wpath(), strError().c_str());
+            }
+            else
+#endif
+            {
+                throw Error(18, path(), strError());
+            }
+        }
+    } // FileIo::transfer
 
     int FileIo::putb(byte data)
     {
@@ -994,16 +1134,16 @@ namespace Exiv2 {
     {
         FileIo file(wpath);
         if (file.open("rb") != 0) {
-            throw Error(10, ws2s(wpath), "rb", strError());
+            throw WError(10, wpath, "rb", strError().c_str());
         }
         struct _stat st;
         if (0 != ::_wstat(wpath.c_str(), &st)) {
-            throw Error(2, ws2s(wpath), strError(), "::_wstat");
+            throw WError(2, wpath, strError().c_str(), "::_wstat");
         }
         DataBuf buf(st.st_size);
         long len = file.read(buf.pData_, buf.size_);
         if (len != buf.size_) {
-            throw Error(2, ws2s(wpath), strError(), "FileIo::read");
+            throw WError(2, wpath, strError().c_str(), "FileIo::read");
         }
         return buf;
     }
@@ -1023,7 +1163,7 @@ namespace Exiv2 {
     {
         FileIo file(wpath);
         if (file.open("wb") != 0) {
-            throw Error(10, ws2s(wpath), "wb", strError());
+            throw WError(10, wpath, "wb", strError().c_str());
         }
         return file.write(buf.pData_, buf.size_);
     }
