@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      cr2image.cpp
-  Version:   $Rev: 1937 $
+  Version:   $Rev: 2067 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   22-Apr-06, ahu: created
 
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: cr2image.cpp 1937 2009-11-27 05:59:23Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: cr2image.cpp 2067 2010-04-12 10:43:24Z cgilles $")
 
 // *****************************************************************************
 // included header files
@@ -154,11 +154,11 @@ namespace Exiv2 {
 
     WriteMethod Cr2Parser::encode(
               Blob&     blob,
-        const byte*     pData,
-              uint32_t  size,
-        const ExifData& exifData,
-        const IptcData& iptcData,
-        const XmpData&  xmpData
+        const byte*     /*pData*/,
+              uint32_t  /*size*/,
+        const ExifData& /*exifData*/,
+        const IptcData& /*iptcData*/,
+        const XmpData&  /*xmpData*/
     )
     {
         /* Todo: Implement me!
