@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    exif.hpp
   @brief   Encoding and decoding of Exif data
-  @version $Rev: 1937 $
+  @version $Rev: 2045 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created
@@ -38,7 +38,7 @@
 
 // + standard includes
 #include <string>
-#include <vector>
+#include <list>
 #include <memory>
 
 // *****************************************************************************
@@ -423,7 +423,7 @@ namespace Exiv2 {
     }; // class ExifThumb
 
     //! Container type to hold all metadata
-    typedef std::vector<Exifdatum> ExifMetadata;
+    typedef std::list<Exifdatum> ExifMetadata;
 
     /*!
       @brief A container for Exif data.  This is a top-level class of the %Exiv2
