@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -24,11 +24,11 @@
            <a href="http://www.compton.nu/panasonic.html">Panasonic MakerNote Information</a> by Tom Hughes,
            Panasonic.pm of <a href="http://www.sno.phy.queensu.ca/~phil/exiftool/">ExifTool</a> by Phil Harvey,
            <a href="http://www.ozhiker.com/electronics/pjmt/jpeg_info/panasonic_mn.html">Panasonic Makernote Format Specification</a> by Evan Hunter.
-  @version $Rev: 1571 $
+  @version $Rev: 2109 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @author  Gilles Caulier (gc)
-           <a href="mailto:caulier dot gilles at kdemail dot net">caulier dot gilles at kdemail dot net</a>
+           <a href="mailto:caulier dot gilles at gmail dot com">caulier dot gilles at gmail dot com</a>
   @date    11-Jun-05, ahu: created
  */
 #ifndef PANASONICMN_HPP_
@@ -55,6 +55,8 @@ namespace Exiv2 {
     public:
         //! Return read-only list of built-in Panasonic tags
         static const TagInfo* tagList();
+        //! Return read-only list of built-in Panasonic RAW image tags (IFD0)
+        static const TagInfo* tagListRaw();
 
         //! @name Print functions for Panasonic %MakerNote tags
         //@{
@@ -65,8 +67,10 @@ namespace Exiv2 {
         //@}
 
     private:
-        //! Tag information
+        //! Makernote tag list
         static const TagInfo tagInfo_[];
+        //! Taglist for IFD0 of Panasonic RAW images
+        static const TagInfo tagInfoRaw_[];
 
     }; // class PanasonicMakerNote
 
