@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    tifffwd_int.hpp
   @brief   Internal TIFF parser related typedefs and forward definitions.
-  @version $Rev: 1937 $
+  @version $Rev: 2045 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    24-Jun-06, ahu: created
@@ -36,6 +36,7 @@
 // + standard includes
 #include <memory>
 #include <stack>
+#include <vector>
 
 // *****************************************************************************
 // Exiv2 namespace extensions
@@ -109,6 +110,9 @@ namespace Exiv2 {
     //! Stack to hold a path from the TIFF root element to a TIFF entry
     typedef std::stack<TiffPathItem> TiffPath;
 
+    //! Type for a list of primary image groups
+    typedef std::vector<uint16_t> PrimaryGroups;
+
 }}                                      // namespace Internal, Exiv2
 
 #endif                                  // #ifndef TIFFFWD_INT_HPP_
