@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      tiffvisitor.cpp
-  Version:   $Rev: 1977 $
+  Version:   $Rev: 2239 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   11-Apr-06, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: tiffvisitor.cpp 1977 2009-12-28 14:47:58Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: tiffvisitor.cpp 2239 2010-05-25 14:53:56Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -197,6 +197,91 @@ namespace Exiv2 {
         findObject(object);
     }
 
+    TiffCopier::TiffCopier(      TiffComponent*  pRoot,
+                                 uint32_t        root,
+                           const TiffHeaderBase* pHeader,
+                           const PrimaryGroups*  pPrimaryGroups)
+        : pRoot_(pRoot),
+          root_(root),
+          pHeader_(pHeader),
+          pPrimaryGroups_(pPrimaryGroups)
+    {
+        assert(pRoot_ != 0);
+        assert(pHeader_ != 0);
+        assert(pPrimaryGroups_ != 0);
+    }
+
+    TiffCopier::~TiffCopier()
+    {
+    }
+
+    void TiffCopier::copyObject(TiffComponent* object)
+    {
+        assert(object != 0);
+
+        if (pHeader_->isImageTag(object->tag(), object->group(), pPrimaryGroups_)) {
+            TiffComponent::AutoPtr clone = object->clone();
+            // Assumption is that the corresponding TIFF entry doesn't exist
+            TiffPath tiffPath;
+            TiffCreator::getPath(tiffPath, object->tag(), object->group(), root_);
+            pRoot_->addPath(object->tag(), tiffPath, pRoot_, clone);
+#ifdef DEBUG
+            ExifKey key(object->tag(), tiffGroupName(object->group()));
+            std::cerr << "Copied " << key << "\n";
+#endif
+        }
+    }
+
+    void TiffCopier::visitEntry(TiffEntry* object)
+    {
+        copyObject(object);
+    }
+
+    void TiffCopier::visitDataEntry(TiffDataEntry* object)
+    {
+        copyObject(object);
+    }
+
+    void TiffCopier::visitImageEntry(TiffImageEntry* object)
+    {
+        copyObject(object);
+    }
+
+    void TiffCopier::visitSizeEntry(TiffSizeEntry* object)
+    {
+        copyObject(object);
+    }
+
+    void TiffCopier::visitDirectory(TiffDirectory* /*object*/)
+    {
+        // Do not copy directories (avoids problems with SubIfds)
+    }
+
+    void TiffCopier::visitSubIfd(TiffSubIfd* object)
+    {
+        copyObject(object);
+    }
+
+    void TiffCopier::visitMnEntry(TiffMnEntry* object)
+    {
+        copyObject(object);
+    }
+
+    void TiffCopier::visitIfdMakernote(TiffIfdMakernote* object)
+    {
+        copyObject(object);
+    }
+
+    void TiffCopier::visitBinaryArray(TiffBinaryArray* object)
+    {
+        copyObject(object);
+    }
+
+    void TiffCopier::visitBinaryElement(TiffBinaryElement* object)
+    {
+        copyObject(object);
+    }
+
     TiffDecoder::TiffDecoder(
         ExifData&            exifData,
         IptcData&            iptcData,
@@ -428,22 +513,30 @@ namespace Exiv2 {
             const IptcData&      iptcData,
             const XmpData&       xmpData,
                   TiffComponent* pRoot,
-                  ByteOrder      byteOrder,
+            const bool           isNewImage,
+            const PrimaryGroups* pPrimaryGroups,
+            const TiffHeaderBase* pHeader,
                   FindEncoderFct findEncoderFct
     )
         : exifData_(exifData),
           iptcData_(iptcData),
           xmpData_(xmpData),
           del_(true),
+          pHeader_(pHeader),
           pRoot_(pRoot),
-          pSourceTree_(0),
-          byteOrder_(byteOrder),
-          origByteOrder_(byteOrder),
+          isNewImage_(isNewImage),
+          pPrimaryGroups_(pPrimaryGroups),
+          pSourceTree_(0),          
           findEncoderFct_(findEncoderFct),
           dirty_(false),
           writeMethod_(wmNonIntrusive)
     {
         assert(pRoot != 0);
+        assert(pPrimaryGroups != 0);
+        assert(pHeader != 0);
+
+        byteOrder_ = pHeader->byteOrder();
+        origByteOrder_ = byteOrder_;
 
         encodeIptc();
         encodeXmp();
@@ -711,6 +804,14 @@ namespace Exiv2 {
         byteOrder_ = boOrig;
     }
 
+    bool TiffEncoder::isImageTag(uint16_t tag, uint16_t group) const
+    {
+        if (!isNewImage_ && pHeader_->isImageTag(tag, group, pPrimaryGroups_)) {
+            return true;
+        }
+        return false;
+    }
+
     void TiffEncoder::encodeTiffComponent(
               TiffEntryBase* object,
         const Exifdatum*     datum
@@ -718,6 +819,10 @@ namespace Exiv2 {
     {
         assert(object != 0);
 
+        // Skip image tags of existing TIFF image - they were copied earlier -
+        // but add and encode image tags of new images (creation)
+        if (isImageTag(object->tag(), object->group())) return;
+
         ExifData::iterator pos = exifData_.end();
         const Exifdatum* ed = datum;
         if (ed == 0) {
@@ -962,6 +1067,7 @@ namespace Exiv2 {
     )
     {
         assert(pRootDir != 0);
+
         writeMethod_ = wmIntrusive;
         pSourceTree_ = pSourceDir;
 
@@ -982,6 +1088,10 @@ namespace Exiv2 {
                 continue;
             }
 
+            // Skip image tags of existing TIFF image - they were copied earlier -
+            // but add and encode image tags of new images (creation)
+            if (isImageTag(i->tag(), group)) continue;
+
             // Assumption is that the corresponding TIFF entry doesn't exist
             TiffPath tiffPath;
             TiffCreator::getPath(tiffPath, i->tag(), group, root);
@@ -1250,6 +1360,16 @@ namespace Exiv2 {
 #endif
                     return;
                 }
+                if (object->newGroup_ + i == Group::subimgX) {
+#ifndef SUPPRESS_WARNINGS
+                    std::cerr << "Warning: "
+                              << "Directory " << tiffGroupName(object->group())
+                              << ", entry 0x" << std::setw(4)
+                              << std::setfill('0') << std::hex << object->tag()
+                              << ": Skipping sub-IFDs beyond the first " << i << ".\n";
+#endif
+                    break;
+                }
                 // If there are multiple dirs, group is incremented for each
                 TiffComponent::AutoPtr td(new TiffDirectory(object->tag(),
                                                             object->newGroup_ + i));
@@ -1442,10 +1562,10 @@ namespace Exiv2 {
 
         if (object->TiffEntryBase::doSize() == 0) return;
         if (!object->initialize(pRoot_)) return;
-        const ArrayDef* defs = object->def();
-        const ArrayDef* def = &object->cfg()->elDefaultDef_;
 
-        const CryptFct cryptFct = object->cfg()->cryptFct_;
+        const ArrayCfg* cfg = object->cfg();
+
+        const CryptFct cryptFct = cfg->cryptFct_;
         if (cryptFct != 0) {
             const byte* pData = object->pData();
             int32_t size = object->TiffEntryBase::doSize();
@@ -1453,12 +1573,41 @@ namespace Exiv2 {
             if (buf.size_ > 0) object->setData(buf);
         }
 
+        const ArrayDef* defs = object->def();
+        const ArrayDef* defsEnd = defs + object->defSize();
+        const ArrayDef* def = &cfg->elDefaultDef_;
+        ArrayDef gap = *def;
+
         for (uint32_t idx = 0; idx < object->TiffEntryBase::doSize(); ) {
             if (defs) {
-                def = std::find(defs, defs + object->defSize(), idx);
-                if (def == defs + object->defSize()) def = &object->cfg()->elDefaultDef_;
+                def = std::find(defs, defsEnd, idx);
+                if (def == defsEnd) {
+                    if (cfg->concat_) {
+                        // Determine gap-size
+                        const ArrayDef* xdef = defs;
+                        for (; xdef != defsEnd && xdef->idx_ <= idx; ++xdef) {}
+                        uint32_t gapSize = 0;
+                        if (xdef != defsEnd && xdef->idx_ > idx) {
+                            gapSize = xdef->idx_ - idx;
+                        }
+                        else {
+                            gapSize = object->TiffEntryBase::doSize() - idx;
+                        }
+                        gap.idx_ = idx;
+                        gap.tiffType_ = cfg->elDefaultDef_.tiffType_;
+                        gap.count_ = gapSize / cfg->tagStep();
+                        if (gap.count_ * cfg->tagStep() != gapSize) {
+                            gap.tiffType_ = ttUndefined;
+                            gap.count_ = gapSize;
+                        }
+                        def = &gap;
+                    }
+                    else {
+                        def = &cfg->elDefaultDef_;
+                    }
+                }
             }
-            idx += object->addElement(idx, def); // idx may be different from def->idx_
+            idx += object->addElement(idx, *def); // idx may be different from def->idx_
         }
 
     } // TiffReader::visitBinaryArray
